<?php
/**
 * The MIMP_IMAP:: class facilitates connections to the IMAP/POP3 server
 * via the c-client PHP extensions.
 *
 * $Horde: mimp/lib/IMAP.php,v 1.10 2006/01/01 21:11:16 jan Exp $
 *
 * Copyright 2003-2006 Michael Slusarz <slusarz@bigworm.colorado.edu>
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
 * @package MIMP
 */
class MIMP_IMAP {

    /**
     * The server string.
     *
     * @var string
     */
    var $_serverString;

    /**
     * The username for the server.
     *
     * @var string
     */
    var $_user;

    /**
     * The password for the mail server.
     *
     * @var string
     */
    var $_pass;

    /**
     * The currently open mailbox.
     *
     * @var string
     */
    var $_openMbox = null;

    /**
     * The IMAP flags set in the currently open mailbox.
     *
     * @var integer
     */
    var $_mboxFlags = null;

    /**
     * Attempts to return a reference to a concrete MIMP_IMAP instance.
     * It will only create a new instance if no MIMP_IMAP instance currently
     * exists.
     *
     * This method must be invoked as:
     *   $mimp_imap = &MIMP_IMAP::singleton();
     *
     * @param array $params  Parameters needed.
     *
     * @return MIMP_IMAP  The concrete MIMP_IMAP reference, or false on error.
     */
    function &singleton($params = array())
    {
        static $instance;

        if (!isset($instance)) {
            $instance = new MIMP_IMAP($params);
        }

        return $instance;
    }

    /**
     * Constructor.
     *
     * @param array $params  Any additional parameters needed.
     */
    function MIMP_IMAP($params = array())
    {
        $this->_serverString = MIMP::serverString($_SESSION['mimp']['protocol']);
        $this->_user = $_SESSION['mimp']['user'];
        $this->_pass = Secret::read(Secret::getKey('mimp'), $_SESSION['mimp']['pass']);
    }

    /**
     * Open an IMAP stream.
     *
     * @param string $mbox    A mailbox to open.
     * @param integer $flags  Any flags that need to be passed to imap_open().
     *
     * @return boolean  The return from the imap_open() call.
     */
    function openIMAPStream($mbox = null, $flags = 0)
    {
        return @imap_open($this->_serverString . $mbox, $this->_user, $this->_pass, $flags);
    }

    /**
     * Change the currently active MIMP IMAP stream to a new mailbox
     * (if necessary).
     *
     * @param string $mbox    The new mailbox.
     * @param integer $flags  Any flags that need to be passed to
     *                        imap_reopen().
     *
     * @return boolean  True on success, false on error.
     */
    function changeMbox($mbox, $flags = null)
    {
        /* Open a connection if none exists. */
        if (!isset($_SESSION['mimp']['stream']) ||
            empty($_SESSION['mimp']['stream'])) {
            if (($_SESSION['mimp']['stream'] = $this->openIMAPStream($mbox, $flags))) {
                $this->_openMbox = $mbox;
                $this->_mboxFlags = $flags;
                return true;
            } else {
                return false;
            }
        }

        if ($_SESSION['mimp']['base_protocol'] == 'pop3') {
            return true;
        }

        /* Only reopen mailbox if we need to - either we are changing
           mailboxes or the flags for the current mailbox has changed. */
        if (($this->_openMbox != $mbox) || ($this->_mboxFlags != $flags)) {
            if (is_null($flags)) {
                $result = @imap_reopen($_SESSION['mimp']['stream'], $this->_serverString . $mbox);
            } else {
                $result = @imap_reopen($_SESSION['mimp']['stream'], $this->_serverString . $mbox, $flags);
            }

            if ($result) {
                $this->_openMbox = $mbox;
                $this->_mboxFlags = $flags;
                return true;
            } else {
                return false;
            }
        }

        return true;
    }

    /**
     * Reopen an IMAP stream.
     *
     * @param boolean $expunge  Expunge before closing?
     *
     * @return mixed  See MIMP::authenticate().
     */
    function reopenIMAPStream($expunge = false)
    {
        $flags = 0;
        if ($expunge) {
            $flags = CL_EXPUNGE;
        }

        @imap_close($_SESSION['mimp']['stream'], $flags);
        unset($_SESSION['mimp']['stream']);
        return MIMP::authenticate();
    }

}
